from typing_extensions import NotRequired, TypedDict


class Tag(TypedDict):
    name: str
    """
    The name of the email tag.
    It can only contain ASCII letters (a–z, A–Z), numbers (0–9),
    underscores (_), or dashes (-).
    It can contain no more than 256 characters.
    """
    value: NotRequired[str]
    """
    The value of the email tag.
    It can only contain ASCII letters (a–z, A–Z), numbers (0–9),
    underscores (_), or dashes (-).
    It can contain no more than 256 characters.
    """



"""Resend Exceptions module.

This module defines the base types for platform-wide error
codes as outlined in https://resend.com/docs/errors.
"""

from typing import Any, Dict, Union


class ResendError(Exception):
    """Base class for all errors raised by Resend SDK.
    This is the parent class of all exceptions (server side)
    raised by the Resend SDK. Developers can simply catch
    this class and inspect its `code` to implement more specific
    error handling. Note that for some client-side errors ie:
    some method argument missing, a ValueError would be raised.

    Args:
        code: A string error indicating the HTTP status code
        attributed to that Error.
        message: A human-readable error message string.
        suggested_action: A suggested action path to help the user.
        error_type: Maps to the `type` field from the Resend API
    """

    def __init__(
        self,
        code: Union[str, int],
        error_type: str,
        message: str,
        suggested_action: str,
    ):
        Exception.__init__(self, message)
        self.code = code
        self.message = message
        self.suggested_action = suggested_action
        self.error_type = error_type


class MissingApiKeyError(ResendError):
    """see https://resend.com/docs/errors"""

    def __init__(
        self,
        message: str,
        error_type: str,
        code: Union[str, int],
    ):
        suggested_action = """Include the following header
        Authorization: Bearer YOUR_API_KEY in the request."""

        message = "Missing API key in the authorization header."

        ResendError.__init__(
            self,
            message=message,
            suggested_action=suggested_action,
            code=code,
            error_type=error_type,
        )


class InvalidApiKeyError(ResendError):
    """see https://resend.com/docs/errors"""

    def __init__(
        self,
        message: str,
        error_type: str,
        code: Union[str, int],
    ):
        suggested_action = """Generate a new API key in the dashboard."""

        ResendError.__init__(
            self,
            message=message,
            suggested_action=suggested_action,
            code=code,
            error_type=error_type,
        )


class ValidationError(ResendError):
    """see https://resend.com/docs/errors"""

    def __init__(
        self,
        message: str,
        error_type: str,
        code: Union[str, int],
    ):
        default_message = """
        The request body is missing one or more required fields."""

        suggested_action = """Check the error message
        to see the list of missing fields."""

        if message == "":
            message = default_message

        ResendError.__init__(
            self,
            code=code or "400",
            message=message,
            suggested_action=suggested_action,
            error_type=error_type,
        )


class MissingRequiredFieldsError(ResendError):
    """see https://resend.com/docs/errors"""

    def __init__(
        self,
        message: str,
        error_type: str,
        code: Union[str, int],
    ):
        default_message = """
        The request body is missing one or more required fields."""

        suggested_action = """Check the error message
        to see the list of missing fields."""

        if message == "":
            message = default_message

        ResendError.__init__(
            self,
            code=code or "422",
            message=message,
            suggested_action=suggested_action,
            error_type=error_type,
        )


class ApplicationError(ResendError):
    """see https://resend.com/docs/errors"""

    def __init__(
        self,
        message: str,
        error_type: str,
        code: Union[str, int],
    ):
        default_message = """
        Something went wrong."""

        suggested_action = """Contact Resend support."""

        if message == "":
            message = default_message

        ResendError.__init__(
            self,
            code=code or "500",
            message=message,
            suggested_action=suggested_action,
            error_type=error_type,
        )


# Dict with error code -> error type mapping
ERRORS: Dict[str, Dict[str, Any]] = {
    "400": {"validation_error": ValidationError},
    "422": {
        "missing_required_fields": MissingRequiredFieldsError,
        "validation_error": ValidationError,
    },
    "401": {"missing_api_key": MissingApiKeyError},
    "403": {"invalid_api_key": InvalidApiKeyError},
    "500": {"application_error": ApplicationError},
}


def raise_for_code_and_type(
    code: Union[str, int], error_type: str, message: str
) -> None:
    """Raise the appropriate error based on the code and type.

    Args:
        code (str): The error code
        error_type (str): The error type
        message (str): The error message

    Raises:
        ResendError: If it is a Resend err
            or
        ValidationError: If the error type is validation_error
            or
        MissingRequiredFieldsError: If the error type is missing_required_fields
            or
        MissingApiKeyError: If the error type is missing_api_key
            or
        InvalidApiKeyError: If the error type is invalid_api_key
            or
        ApplicationError: If the error type is application_error
            or
        TypeError: If the error type is not found
    """
    error = ERRORS.get(str(code))

    # Handle the case where the error might be unknown
    if error is None:
        raise ResendError(
            code=code, message=message, error_type=error_type, suggested_action=""
        )

    # Raise error from errors list
    error_from_list = error.get(error_type)

    if error_from_list is not None:
        raise error_from_list(
            code=code,
            message=message,
            error_type=error_type,
        )
    # defaults to ResendError if finally can't find error type
    raise ResendError(
        code=code, message=message, error_type=error_type, suggested_action=""
    )



from typing import Any, Dict, List, cast

from typing_extensions import NotRequired, TypedDict

from resend import request

from ._contact import Contact


class Contacts:
    class CreateParams(TypedDict):
        audience_id: str
        """
        The audience id.
        """
        email: str
        """
        The email of the contact.
        """
        first_name: NotRequired[str]
        """
        The first name of the contact.
        """
        last_name: NotRequired[str]
        """
        The last name of the contact.
        """
        unsubscribed: NotRequired[bool]
        """
        The unsubscribed status of the contact.
        """

    class UpdateParams(TypedDict):
        audience_id: str
        """
        The audience id.
        """
        id: str
        """
        The contact id.
        """
        email: NotRequired[str]
        """
        The email of the contact.
        """
        first_name: NotRequired[str]
        """
        The first name of the contact.
        """
        last_name: NotRequired[str]
        """
        The last name of the contact.
        """
        unsubscribed: NotRequired[bool]
        """
        The unsubscribed status of the contact.
        """

    @classmethod
    def create(cls, params: CreateParams) -> Contact:
        """
        Create a new contact.
        see more: https://resend.com/docs/api-reference/contacts/create-contact

        Args:
            params (CreateParams): The contact creation parameters

        Returns:
            Contact: The new contact object
        """
        path = f"/audiences/{params['audience_id']}/contacts"
        return Contact.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="post"
            ).perform()
        )

    @classmethod
    def update(cls, params: UpdateParams) -> Contact:
        """
        Update an existing contact.
        see more: https://resend.com/docs/api-reference/contacts/update-contact

        Args:
            params (UpdateParams): The contact update parameters

        Returns:
            Contact: The updated contact object
        """
        path = f"/audiences/{params['audience_id']}/contacts/{params['id']}"
        return Contact.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="patch"
            ).perform()
        )

    @classmethod
    def list(cls, audience_id: str) -> List[Contact]:
        """
        List all contacts for the provided audience.
        see more: https://resend.com/docs/api-reference/contacts/list-contacts

        Args:
            audience_id (str): The audience ID

        Returns:
            List[Contact]: A list of contact objects
        """
        path = f"/audiences/{audience_id}/contacts"
        resp = request.Request(path=path, params={}, verb="get").perform()
        return (
            [Contact.new_from_request(contact) for contact in resp["data"]]
            if "data" in resp
            else []
        )

    @classmethod
    def get(cls, id: str, audience_id: str) -> Contact:
        """
        Get a contact.
        see more: https://resend.com/docs/api-reference/contacts/get-contact

        Args:
            id (str): The contact ID
            audience_id (str): The audience ID

        Returns:
            Contact: The contact object
        """
        path = f"/audiences/{audience_id}/contacts/{id}"
        return Contact.new_from_request(
            request.Request(path=path, params={}, verb="get").perform()
        )

    @classmethod
    def remove(cls, audience_id: str, id: str = "", email: str = "") -> Contact:
        """
        Remove a contact by ID or by Email
        see more: https://resend.com/docs/api-reference/contacts/delete-contact

        Args:
            audience_id (str): The audience ID
            id (str): The contact ID
            email (str): The contact email

        Returns:
            Contact: The removed contact object
        """
        contact = email if id == "" else id
        if contact == "":
            raise ValueError("id or email must be provided")
        path = f"/audiences/{audience_id}/contacts/{contact}"

        return Contact.new_from_request(
            request.Request(path=path, params={}, verb="delete").perform()
        )



from typing import Any, Dict, List, Union


class Email:
    id: str
    """
    The Email ID.
    """
    to: Union[List[str], str]
    """
    List of email addresses to send the email to.
    """
    from_: str
    """
    The email address of the sender.
    "from" is a reserved keyword in python.
    So accept either "from_" or "sender"
    """
    sender: str
    """
    The email address of the sender. "from" is a reserved keyword in python.
    So accept either "from_" or "sender"
    """
    created_at: str
    """
    When the email was created.
    """
    subject: str
    """
    The subject of the email.
    """
    html: str
    """
    The HTML content of the email.
    """
    text: str
    """
    The text content of the email.
    """
    bcc: Union[List[str], str]
    """
    Bcc
    """
    cc: Union[List[str], str]
    """
    Cc
    """
    reply_to: Union[List[str], str]
    """
    Reply to
    """
    last_event: str
    """
    The last event of the email.
    """

    def __init__(
        self,
        id: str,
        to: Union[List[str], str],
        from_: str,
        sender: str,
        created_at: str,
        subject: str,
        html: str,
        text: str,
        bcc: Union[List[str], str],
        cc: Union[List[str], str],
        reply_to: Union[List[str], str],
        last_event: str,
    ):
        self.id = id
        self.to = to
        self.from_ = from_
        self.sender = sender
        self.created_at = created_at
        self.subject = subject
        self.html = html
        self.text = text
        self.bcc = bcc
        self.cc = cc
        self.reply_to = reply_to
        self.last_event = last_event

    @staticmethod
    def new_from_request(val: Dict[Any, Any]) -> "Email":
        """Creates a new Email object from the
        JSON response from the API.

        Args:
            val (Dict): The JSON response from the API

        Returns:
            Email: The new Email object
        """
        email = Email(
            id=val["id"] if "id" in val else "",
            to=val["to"] if "to" in val else "",
            # we set sender as the value from "from" here
            # because "from" is a reserved keyword in python
            from_=val["from"] if "from" in val else "",
            sender=val["from"] if "from" in val else "",
            created_at=val["created_at"] if "created_at" in val else "",
            subject=val["subject"] if "subject" in val else "",
            html=val["html"] if "html" in val else "",
            text=val["text"] if "text" in val else "",
            bcc=val["bcc"] if "bcc" in val else "",
            cc=val["cc"] if "cc" in val else "",
            reply_to=val["reply_to"] if "reply_to" in val else "",
            last_event=val["last_event"] if "last_event" in val else "",
        )
        return email



from typing import Any, Dict, List, cast

from typing_extensions import NotRequired, TypedDict

from resend import request
from resend.api_keys._api_key import ApiKey


class ApiKeys:
    class CreateParams(TypedDict):
        name: str
        """
        The API key name.
        """
        permission: NotRequired[str]
        """
        The API key can have full access to Resend's API or be only
        restricted to send emails. * full_access: Can create, delete, get,
        and update any resource. * sending_access: Can only send emails.
        """
        domain_id: NotRequired[str]
        """
        Restrict an API key to send emails only from a specific domain.
        This is only used when the permission is set to sending_access.
        """

    @classmethod
    def create(cls, params: CreateParams) -> ApiKey:
        """
        Add a new API key to authenticate communications with Resend.
        see more: https://resend.com/docs/api-reference/api-keys/create-api-key

        Args:
            params (CreateParams): The API key creation parameters

        Returns:
            ApiKey: The new API key object
        """
        path = "/api-keys"
        return ApiKey.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="post"
            ).perform()
        )

    @classmethod
    def list(cls) -> List[ApiKey]:
        """
        Retrieve a list of API keys for the authenticated user.
        see more: https://resend.com/docs/api-reference/api-keys/list-api-keys

        Returns:
            List[ApiKey]: A list of API key objects
        """
        path = "/api-keys"
        resp = request.Request(path=path, params={}, verb="get").perform()
        return [ApiKey.new_from_request(val) for val in resp["data"]]

    @classmethod
    def remove(cls, api_key_id: str) -> None:
        """
        Remove an existing API key.
        see more: https://resend.com/docs/api-reference/api-keys/delete-api-key

        Args:
            api_key_id (str): The ID of the API key to remove

        Returns:
            None
        """
        path = f"/api-keys/{api_key_id}"

        # This would raise if failed
        request.Request(path=path, params={}, verb="delete").perform()
        return None



from typing import Any, Dict, List, cast

from resend import request
from resend.utils import replace_params

from ._email import Email
from ._emails import Emails


class Batch:
    @classmethod
    def send(cls, params: List[Emails.SendParams]) -> List[Email]:
        """
        Trigger up to 100 batch emails at once.
        see more: https://resend.com/docs/api-reference/emails/send-batch-emails

        Args:
            params (List[Emails.SendParams]): The list of emails to send

        Returns:
            List[Email]: A list of email objects
        """
        path = "/emails/batch"

        # loop through the list of params and replace "from_" or "sender" with "from"
        replaced_params = [
            replace_params(cast(Dict[Any, Any], param)) for param in params
        ]

        resp = request.Request(path=path, params=replaced_params, verb="post").perform()
        return [Email.new_from_request(val) for val in resp["data"]]



from typing import Any, Dict


# we need this workaround here because "from" is a reserved keyword
# in python, so accept either "sender" or "from_" in the SendParams class
def replace_params(params: Dict[Any, Any]) -> Dict[Any, Any]:
    if "from_" in params:
        params["from"] = params["from_"]
        params.pop("from_")
    elif "sender" in params:
        params["from"] = params["sender"]
        params.pop("sender")
    return params



import os

from .api_keys._api_key import ApiKey
from .api_keys._api_keys import ApiKeys
from .audiences._audience import Audience
from .audiences._audiences import Audiences
from .contacts._contact import Contact
from .contacts._contacts import Contacts
from .domains._domain import Domain
from .domains._domains import Domains
from .emails._attachment import Attachment
from .emails._batch import Batch
from .emails._email import Email
from .emails._emails import Emails
from .emails._tag import Tag
from .request import Request
from .version import get_version

# Config vars
api_key = os.environ.get("RESEND_API_KEY")
api_url = os.environ.get("RESEND_API_URL", "https://api.resend.com")

# API resources
from .emails._emails import Emails  # noqa

__all__ = [
    "get_version",
    "Request",
    "Emails",
    "ApiKeys",
    "Domains",
    "Batch",
    "Audiences",
    "Contacts",
    # Types
    "Audience",
    "Contact",
    "Domain",
    "ApiKey",
    "Email",
    "Attachment",
    "Tag",
]



#!/usr/bin/env python

from setuptools import find_packages, setup

from resend.version import get_version

install_requires = open("requirements.txt").readlines()

setup(
    name="resend",
    version=get_version(),
    description="Resend Python SDK",
    long_description=open("README.md", encoding="utf8").read(),
    long_description_content_type="text/markdown",
    author="Derich Pacheco",
    author_email="carlosderich@gmail.com",
    url="https://github.com/resendlabs/resend-python",
    packages=find_packages(exclude=["tests", "tests.*"]),
    package_data={"resend": ["py.typed"]},
    install_requires=install_requires,
    zip_safe=False,
    python_requires=">=3.7",
    keywords=["email", "email platform"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)
